@@ -12,9 +12,6 @@ use gst::prelude::*;
 use gst::subclass::prelude::*;
 use gst_sdp::SDPMessage;
 use gst_webrtc::{WebRTCICEGatheringState, WebRTCSessionDescription};
-use reqwest::header::HeaderMap;
-use reqwest::header::HeaderValue;
-use reqwest::StatusCode;
 use std::collections::HashMap;
 use std::sync::LazyLock;
 use std::sync::Mutex;
@@ -22,14 +19,7 @@ use std::sync::Mutex;
 use std::net::SocketAddr;
 use tokio::sync::mpsc;
 use url::Url;
-use warp::{
-    http,
-    hyper::{
-        header::{CONTENT_TYPE, LINK},
-        Body,
-    },
-    Filter, Reply,
-};
+use warp::{http, hyper::Body, Filter, Reply};
 
 static CAT: LazyLock<gst::DebugCategory> = LazyLock::new(|| {
     gst::DebugCategory::new(
@@ -189,17 +179,17 @@ impl WhipClient {
         let body = sdp.as_text().unwrap();
 
         gst::debug!(CAT, imp = self, "Using endpoint {}", endpoint.as_str());
-        let mut headermap = HeaderMap::new();
+        let mut headermap = reqwest::header::HeaderMap::new();
         headermap.insert(
             reqwest::header::CONTENT_TYPE,
-            HeaderValue::from_static(CONTENT_SDP),
+            reqwest::header::HeaderValue::from_static(CONTENT_SDP),
         );
 
         if let Some(token) = auth_token.as_ref() {
             let bearer_token = "Bearer ".to_owned() + token;
             headermap.insert(
                 reqwest::header::AUTHORIZATION,
-                HeaderValue::from_str(bearer_token.as_str())
+                reqwest::header::HeaderValue::from_str(bearer_token.as_str())
                     .expect("Failed to set auth token to header"),
             );
         }
@@ -243,7 +233,7 @@ impl WhipClient {
         gst::debug!(CAT, "response status: {}", resp.status());
 
         match resp.status() {
-            StatusCode::OK | StatusCode::CREATED => {
+            reqwest::StatusCode::OK | reqwest::StatusCode::CREATED => {
                 if use_link_headers {
                     if let Err(e) = set_ice_servers(webrtcbin, resp.headers()) {
                         self.raise_error(e.to_string());
@@ -398,12 +388,12 @@ impl WhipClient {
 
         drop(state);
 
-        let mut headermap = HeaderMap::new();
+        let mut headermap = reqwest::header::HeaderMap::new();
         if let Some(token) = &settings.auth_token {
             let bearer_token = "Bearer ".to_owned() + token.as_str();
             headermap.insert(
                 reqwest::header::AUTHORIZATION,
-                HeaderValue::from_str(bearer_token.as_str())
+                reqwest::header::HeaderValue::from_str(bearer_token.as_str())
                     .expect("Failed to set auth token to header"),
             );
         }
@@ -746,12 +736,15 @@ impl WhipServer {
         let settings = self.settings.lock().unwrap();
         drop(settings);
 
-        let mut links = HeaderMap::new();
+        let mut links = http::HeaderMap::new();
         let settings = self.settings.lock().unwrap();
         match &settings.stun_server {
             Some(stun) => match build_link_header(stun.as_str()) {
                 Ok(stun_link) => {
-                    links.append(LINK, HeaderValue::from_str(stun_link.as_str()).unwrap());
+                    links.append(
+                        http::header::LINK,
+                        warp::http::HeaderValue::from_str(stun_link.as_str()).unwrap(),
+                    );
                 }
                 Err(e) => {
                     gst::error!(CAT, imp = self, "Failed to parse {stun:?} : {e:?}");
@@ -766,7 +759,10 @@ impl WhipServer {
                     gst::debug!(CAT, imp = self, "turn server: {}", turn.as_str());
                     match build_link_header(turn.as_str()) {
                         Ok(turn_link) => {
-                            links.append(LINK, HeaderValue::from_str(turn_link.as_str()).unwrap());
+                            links.append(
+                                http::header::LINK,
+                                warp::http::HeaderValue::from_str(turn_link.as_str()).unwrap(),
+                            );
                         }
                         Err(e) => {
                             gst::error!(CAT, imp = self, "Failed to parse {turn_server:?} : {e:?}");
@@ -796,8 +792,19 @@ impl WhipServer {
     async fn post_handler(
         &self,
         body: warp::hyper::body::Bytes,
+        id: Option<String>,
     ) -> Result<http::Response<warp::hyper::Body>, warp::Rejection> {
-        let session_id = uuid::Uuid::new_v4().to_string();
+        let session_id = match id {
+            Some(id) => {
+                gst::debug!(CAT, imp = self, "got session id {id} from the URL");
+                id
+            }
+            None => {
+                gst::info!(CAT, imp = self, "no session id in the URL, generating UUID");
+                uuid::Uuid::new_v4().to_string()
+            }
+        };
+
         let (tx, mut rx) = mpsc::channel::<Option<SDPMessage>>(1);
         let wait_timeout = {
             let mut settings = self.settings.lock().unwrap();
@@ -860,13 +867,16 @@ impl WhipServer {
         };
 
         let settings = self.settings.lock().unwrap();
-        let mut links = HeaderMap::new();
+        let mut links = http::HeaderMap::new();
 
         #[allow(clippy::single_match)]
         match &settings.stun_server {
             Some(stun) => match build_link_header(stun.as_str()) {
                 Ok(stun_link) => {
-                    links.append(LINK, HeaderValue::from_str(stun_link.as_str()).unwrap());
+                    links.append(
+                        http::header::LINK,
+                        http::HeaderValue::from_str(stun_link.as_str()).unwrap(),
+                    );
                 }
                 Err(e) => {
                     gst::error!(CAT, imp = self, "Failed to parse {stun:?} : {e:?}");
@@ -881,7 +891,10 @@ impl WhipServer {
                     gst::debug!(CAT, imp = self, "turn server: {}", turn.as_str());
                     match build_link_header(turn.as_str()) {
                         Ok(turn_link) => {
-                            links.append(LINK, HeaderValue::from_str(turn_link.as_str()).unwrap());
+                            links.append(
+                                http::header::LINK,
+                                http::HeaderValue::from_str(turn_link.as_str()).unwrap(),
+                            );
                         }
                         Err(e) => {
                             gst::error!(
@@ -936,8 +949,8 @@ impl WhipServer {
         // Got SDP answer, send answer in the response
         let resource_url = "/".to_owned() + ROOT + "/" + RESOURCE_PATH + "/" + &session_id;
         let mut res = http::Response::builder()
-            .status(StatusCode::CREATED)
-            .header(CONTENT_TYPE, CONTENT_SDP)
+            .status(http::StatusCode::CREATED)
+            .header(http::header::CONTENT_TYPE, CONTENT_SDP)
             .header("location", resource_url)
             .body(Body::from(ans_text.unwrap()))
             .unwrap();
@@ -975,13 +988,33 @@ impl WhipServer {
         let post_filter = warp::post()
             .and(warp::path(ENDPOINT_PATH))
             .and(warp::path::end())
-            .and(warp::header::exact(CONTENT_TYPE.as_str(), CONTENT_SDP))
+            .and(warp::header::exact(
+                http::header::CONTENT_TYPE.as_str(),
+                CONTENT_SDP,
+            ))
             .and(warp::body::bytes())
             .and_then(glib::clone!(
                 #[weak(rename_to = self_)]
                 self,
                 #[upgrade_or_panic]
-                move |body| async move { self_.post_handler(body).await }
+                move |body| async move { self_.post_handler(body, None).await }
+            ));
+
+        // POST /endpoint/<session-id>
+        let post_filter_with_id = warp::post()
+            .and(warp::path(ENDPOINT_PATH))
+            .and(warp::path::param::<String>())
+            .and(warp::path::end())
+            .and(warp::header::exact(
+                http::header::CONTENT_TYPE.as_str(),
+                CONTENT_SDP,
+            ))
+            .and(warp::body::bytes())
+            .and_then(glib::clone!(
+                #[weak(rename_to = self_)]
+                self,
+                #[upgrade_or_panic]
+                move |id, body| async move { self_.post_handler(body, Some(id)).await }
             ));
 
         // OPTIONS /endpoint
@@ -1001,7 +1034,7 @@ impl WhipServer {
             .and(warp::path::param::<String>())
             .and(warp::path::end())
             .and(warp::header::exact(
-                CONTENT_TYPE.as_str(),
+                http::header::CONTENT_TYPE.as_str(),
                 CONTENT_TRICKLE_ICE,
             ))
             .and_then(glib::clone!(
@@ -1024,7 +1057,7 @@ impl WhipServer {
             ));
 
         let api = prefix
-            .and(post_filter)
+            .and(post_filter.or(post_filter_with_id))
             .or(prefix.and(options_filter))
             .or(prefix.and(patch_filter))
             .or(prefix.and(delete_filter));
